#ifndef MSP_GL_POSTPROCESSOR_H_
#define MSP_GL_POSTPROCESSOR_H_

#include <msp/datafile/objectloader.h>

namespace Msp {
namespace GL {

class Mesh;
class Renderer;
class Resources;
class Sampler;
class Shader;
class Texture2D;

/**
Base class for post-processing effects.  Post-processors receive the contents
of the entire framebuffer as a texture and render it back, altering it in the
process.
*/
class PostProcessor
{
public:
	struct Template
	{
		class Loader: public Msp::DataFile::ObjectLoader<Template>
		{
		public:
			Loader(Template &);
		};

		unsigned size_divisor;

		Template();
		virtual ~Template() { }

		virtual PostProcessor *create(Resources &, unsigned, unsigned) const = 0;
	};

protected:
	PostProcessor() { }
public:
	virtual ~PostProcessor() { }

	/// Renders the effect.
	virtual void render(const Texture2D &, const Texture2D &) { }

	virtual void render(Renderer &, const Texture2D &, const Texture2D &);
};

} // namespace GL
} // namespace Msp

#endif
